package com.utopiaindustries.hseobservationsapp.activities.HSETrainingForms;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.EditorInfo;
import android.widget.AdapterView;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;

import com.utopiaindustries.hseobservationsapp.R;

import com.utopiaindustries.hseobservationsapp.adapters.SafetyTrainingAdapter;
import com.utopiaindustries.hseobservationsapp.helper.Helper;
import com.utopiaindustries.hseobservationsapp.models.HseData.HseSafetyTrainingTopic;
import com.utopiaindustries.hseobservationsapp.utils.StorageManager.StorageManager;

import java.util.ArrayList;

public class HseOneActivity extends AppCompatActivity {


    String docTypeId = "";
    String docTypeTitle = "";

    ImageView imgBack;
    Button btnNext;
    AutoCompleteTextView safetyTextview;
    EditText etTrainingSession, etDescription;

    private ArrayList<HseSafetyTrainingTopic> safetyTrainingArrayList = new ArrayList<>();

    private SafetyTrainingAdapter safetyTrainingAdapter;

    private String safetyTrainingTopic = "";
    EditText etNoOfEmployees;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        setContentView(R.layout.activity_hse_one);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });

        initializeLayouts();

        imgBack.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                alertExit(HseOneActivity.this);
            }
        });

        btnNext.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isValidate()) {
                    Intent intent = new Intent(HseOneActivity.this, HseTwoActivity.class);
                    startActivity(intent);
                    overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left);
                }

            }
        });

        safetyTextview.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                safetyTrainingTopic = safetyTrainingArrayList.get(position).getTitle();
                StorageManager.getInstance().getHseTrainingModel().get(0).setSafetyTopicId(safetyTrainingArrayList.get(position).getId());
                StorageManager.getInstance().getHseTrainingModel().get(0).setSafetyTopicName(safetyTrainingArrayList.get(position).getTitle());
            }
        });

        etNoOfEmployees.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                if (!s.toString().isEmpty()) {
                    StorageManager.getInstance().getHseTrainingModel().get(0).setEmployeeAttendance(Integer.parseInt(s.toString()));
                }

            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        etDescription.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                StorageManager.getInstance().getHseTrainingModel().get(0).setDescription(s.toString());
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        etTrainingSession.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                if (!s.toString().isEmpty()) {
                    StorageManager.getInstance().getHseTrainingModel().get(0).setTrainingMinutes(Double.parseDouble(s.toString()));
                }
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });
    }

    @Override
    public void onBackPressed() {

        alertExit(HseOneActivity.this);
    }

    private void initializeLayouts() {
        imgBack = findViewById(R.id.img_back);
        btnNext = findViewById(R.id.btn_next);
        etNoOfEmployees = findViewById(R.id.et_no_of_employees);
        etTrainingSession = findViewById(R.id.et_training_session);
        etDescription = findViewById(R.id.et_description);
        etDescription.setImeOptions(EditorInfo.IME_ACTION_DONE);
        etDescription.setRawInputType(InputType.TYPE_CLASS_TEXT);

        safetyTextview = findViewById(R.id.safety_textview);

        safetyTrainingArrayList.addAll(Helper.getList(Helper.hseSafetyTrainingTopics, this, HseSafetyTrainingTopic.class));
        safetyTrainingAdapter = new SafetyTrainingAdapter(this, safetyTrainingArrayList);
        safetyTextview.setAdapter(safetyTrainingAdapter);

    }

    public void alertExit(Context con) {
        ViewGroup viewGroup = findViewById(android.R.id.content);

        TextView dialogOkBtn, dialogCancelBtn;

        AlertDialog.Builder builder = new AlertDialog.Builder(con);
        View view1 = LayoutInflater.from(con).inflate(R.layout.custom_layout_for_exit, viewGroup, false);
        builder.setCancelable(false);
        builder.setView(view1);

        dialogOkBtn = view1.findViewById(R.id.dialogOkBtn);
        dialogCancelBtn = view1.findViewById(R.id.dialogCancelBtn);

        AlertDialog alertDialog = builder.create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        dialogOkBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                alertDialog.dismiss();

                StorageManager.getInstance().clearHseTrainingModule();
                finish();

            }
        });

        dialogCancelBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                alertDialog.dismiss();

            }
        });

        alertDialog.show();
    }

    /*@AfterPermissionGranted(CAMERA_REQUEST_ITEM)
    public void openItemCamera() {
        if (hasCameraPermission()) {
            try {
                Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
                File photoFile = null;
                try {
                    photoFile = createImageFile("Item");
                } catch (IOException ex) {
                    // Error occurred while creating the File
                }

                if (photoFile != null) {
                    Uri photoURI = FileProvider.getUriForFile(this,
                            "com.utopia.inventorymanagement",
                            photoFile);
                    takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, photoURI);
                    //startActivityForResult(takePictureIntent, CAMERA_REQUEST);
                    itemCameraLauncher.launch(takePictureIntent);
                }
            } catch (ActivityNotFoundException e) {
                Toast.makeText(this, "Camera app not found", Toast.LENGTH_SHORT).show();
            }
        } else {
            // Ask for one permission
            String[] perms = {};
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                perms = new String[]{Manifest.permission.READ_MEDIA_IMAGES, Manifest.permission.CAMERA};
            } else {
                perms = new String[]{Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.CAMERA};
            }

            EasyPermissions.requestPermissions(this, getString(R.string.rationale_camera), CAMERA_REQUEST_ITEM, perms);
        }
    }*/


    public boolean isValidate() {
        boolean returnValue = true;
        String message = "";

        if (etDescription.getText().toString().isEmpty()) {
            message = "Please enter Description";
            returnValue = false;
        }

        if (etTrainingSession.getText().toString().isEmpty()) {
            message = "Please enter Minutes of Session.";
            returnValue = false;
        }

        if (etNoOfEmployees.getText().toString().isEmpty()) {
            message = "Please enter No of Employees.";
            returnValue = false;
        }

        if (safetyTrainingTopic.isEmpty()) {
            message = "Please select Safety Training Topic.";
            returnValue = false;
        }

        if (!returnValue) {
            Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
        }

        return returnValue;
    }

    @Override
    protected void onResume() {
        super.onResume();

        if (StorageManager.getInstance().getHseTrainingModel() != null &&
                !StorageManager.getInstance().getHseTrainingModel().isEmpty()) {
            showPersistData();
        }
    }

    private void showPersistData() {
        safetyTextview.post(() -> safetyTextview.setText(StorageManager.getInstance().getHseTrainingModel().get(0).getSafetyTopicName(), false));

        if (StorageManager.getInstance().getHseTrainingModel().get(0).getEmployeeAttendance() != 0) {
            etNoOfEmployees.setText(String.valueOf(StorageManager.getInstance().getHseTrainingModel().get(0).getEmployeeAttendance()));
        }

        if (StorageManager.getInstance().getHseTrainingModel().get(0).getTrainingMinutes() != 0.0) {
            etTrainingSession.setText(String.valueOf(StorageManager.getInstance().getHseTrainingModel().get(0).getTrainingMinutes()));
        }

        etDescription.setText(StorageManager.getInstance().getHseTrainingModel().get(0).getDescription());
    }
}