package com.utopiaindustries.hseobservationsapp.activities;

import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;
import androidx.lifecycle.ViewModelProvider;

import com.utopiaindustries.hseobservationsapp.R;
import com.utopiaindustries.hseobservationsapp.apiservice.ApiService;
import com.utopiaindustries.hseobservationsapp.apiservice.ApiServiceFactory;
import com.utopiaindustries.hseobservationsapp.apiservice.GiteaClient;
import com.utopiaindustries.hseobservationsapp.apiservice.GiteaRestService;
import com.utopiaindustries.hseobservationsapp.helper.Helper;
import com.utopiaindustries.hseobservationsapp.helper.Preference;
import com.utopiaindustries.hseobservationsapp.helper.PropertyReader;
import com.utopiaindustries.hseobservationsapp.helper.Release;
import com.utopiaindustries.hseobservationsapp.utils.ProgressDialogFragment;
import com.utopiaindustries.hseobservationsapp.utils.StorageManager.StorageManager;
import com.utopiaindustries.hseobservationsapp.viewmodels.LoginViewModel;

import java.util.Properties;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class LoginActivity extends AppCompatActivity {

    EditText tfEmail, tfPassword;
    Button btnLogin;
    LoginViewModel loginViewModel;
    ApiService apiService;
    TextView txtVersion;
    ImageView imgCheckUpdate;
    private String currentVersionName;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        setContentView(R.layout.activity_login);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });

        if (!Helper.isNetworkConnected(this)) {
            Toast.makeText(this, "No Internet Connection", Toast.LENGTH_LONG).show();
        }

        initializeLayout();

        btnLogin.setOnClickListener(v -> {
            if (isValidate()) {
                loginViewModel.isUserAuthenticated(tfEmail.getText().toString().trim(),
                        tfPassword.getText().toString(),
                        new String[]{"ROLE_UIM_HSE_APP_ACCESS_YES"});

                /*Preference.setMyBooleanPref(Helper.project_file, "isLoggedIn", getApplicationContext(), true);
                Preference.setMyStringPref(Helper.project_file,Helper.logInUser,this,tfEmail.getText().toString());

                Intent intent = new Intent(this, DashboardActivity.class);
                startActivity(intent);
                overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left);
                finish();*/
            }
        });

        imgCheckUpdate.setOnClickListener(v -> {
            if (!Helper.isNetworkConnected(this)) {
                Toast.makeText(this, "No Internet Connection", Toast.LENGTH_LONG).show();
            }
            else {
                checkForUpdates();
            }
        });
    }

    public void setAppVersion() {
        PackageManager packageManager = getPackageManager();
        try {
            PackageInfo info = packageManager.getPackageInfo( getPackageName(), 0 );
            txtVersion.setText(String.format("Version: %s", info.versionName));

            String currentVersion = String.format("v-%s", String.valueOf(info.versionName));
            StorageManager.getInstance().setAppVersion(currentVersion);

        } catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    public void initializeLayout() {
        tfEmail = findViewById(R.id.tf_email);
        tfPassword = findViewById(R.id.tf_password);
        btnLogin = findViewById(R.id.btn_login);
        txtVersion = findViewById(R.id.txt_version);
        imgCheckUpdate = findViewById(R.id.img_check_update);
        apiService = ApiServiceFactory.getApiService();

        loginViewModel = new ViewModelProvider(this).get(LoginViewModel.class);

        loginViewModel.getLoadingState().observe(this, isLoading -> {
            if (isLoading != null && isLoading) {
                showProgressDialog();
            } else {
                dismissProgressDialog();
            }
        });

        loginViewModel.getErrorMessage().observe(this, errorResponse -> {
            if (errorResponse.isEmpty()) {
                Toast.makeText(this, "Something went wrong", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(this, errorResponse, Toast.LENGTH_SHORT).show();
            }

        });

        loginViewModel.getLoginUser().observe(this, loginUser -> {
            if (loginUser) {
                Preference.setMyBooleanPref(Helper.project_file, "isLoggedIn", getApplicationContext(), true);
                Preference.setMyStringPref(Helper.project_file,Helper.logInUser,this,tfEmail.getText().toString());

                Intent intent = new Intent(this, DashboardActivity.class);
                startActivity(intent);
                finish();
                overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left);
            }
            else {
                Toast.makeText(this, "Login Failed", Toast.LENGTH_SHORT).show();
            }
        });

        setAppVersion();

        /*loginViewModel.getUserLiveData().observe(this, user -> {
            if (user != null) {
                Preference.setMyBooleanPref(Helper.project_file, "isLoggedIn", getApplicationContext(), true);
                Helper.setPreferenceObject(getApplicationContext(), user, "DriverResponse");

                //Toast.makeText(this, "Welcome " + user.getTruckerName(), Toast.LENGTH_SHORT).show();
                Intent intent = new Intent(this, HomeActivity.class);
                startActivity(intent);
                overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left);
                finish();
            } else {
                Toast.makeText(this, "Login Failed", Toast.LENGTH_SHORT).show();
            }
        });*/
    }

    public boolean isValidate() {
        boolean returnValue = true;
        String message = "";

        if (tfPassword.getText().toString().isEmpty()) {
            message = "Please enter password.";
            returnValue = false;
        }

        /*if (!Helper.isValidEmail(tfEmail.getText().toString())) {
            message = "Please enter valid email.";
            returnValue = false;
        }*/

        if (tfEmail.getText().toString().isEmpty()) {
            message = "Please enter user name.";
            returnValue = false;
        }

        if (!returnValue) {
            Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
        }

        return returnValue;
    }

    public void showProgressDialog() {
        ProgressDialogFragment progressDialog = new ProgressDialogFragment();
        progressDialog.setCancelable(false);
        progressDialog.show(getSupportFragmentManager(), "progressDialog");
    }

    public void dismissProgressDialog() {
        ProgressDialogFragment progressDialog = (ProgressDialogFragment) getSupportFragmentManager().findFragmentByTag("progressDialog");
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
    }

    /**
     * check for updates
     */
    private void checkForUpdates() {
        PackageManager packageManager = getPackageManager();
        try {
            PackageInfo info = packageManager.getPackageInfo( getPackageName(), 0 );
            currentVersionName = info.versionName;

        } catch ( Exception e ) {
            e.printStackTrace();
        }
        // adjust views
        showProgressDialog();
        // get properties
        Properties properties = PropertyReader.getProperties( LoginActivity.this );
        String token = properties.getProperty( "gitea_access_token" );
        // send request
        GiteaRestService restService = GiteaClient.getInstance().create( GiteaRestService.class );
        Call<Release> call = restService.getLatestRelease( token );
        call.enqueue( new Callback<Release>() {
            @Override
            public void onResponse( Call<Release> call, Response<Release> response ) {
                if ( response.body() != null ) {
                    Release release = response.body();
                    try {
                        float releaseVersion = Float.parseFloat( release.getName().replace( "v", "" ) );
                        float currentVersion = Float.parseFloat( currentVersionName.replace( "v", "" ) );
                        if ( releaseVersion > currentVersion && !Helper.isNullOrEmpty( release.getDownloadUrl() ) ) {
                            String msg = String.format( "Updating to version %s", release.getName() );
                            Toast.makeText( LoginActivity.this, msg, Toast.LENGTH_LONG ).show();
                            Intent intent = new Intent( Intent.ACTION_VIEW, Uri.parse( release.getDownloadUrl() ) );
                            startActivity( intent );
                            //startApkDownloadAndPerformUpdate( release.getDownloadUrl(), release.getName() );
                        } else {
                            Toast.makeText( LoginActivity.this, "Latest version is already installed", Toast.LENGTH_LONG ).show();
                        }
                    } catch ( Exception e ) {
                        Toast.makeText( LoginActivity.this, e.getMessage(), Toast.LENGTH_LONG ).show();
                        e.printStackTrace();
                    }
                }
                dismissProgressDialog();
            }

            @Override
            public void onFailure( Call<Release> call, Throwable t ) {
                dismissProgressDialog();
                t.printStackTrace();
            }
        } );
    }
}