package com.utopiaindustries.hseobservationsapp.activities.ObservationForms;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.EditorInfo;
import android.widget.AdapterView;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;
import androidx.lifecycle.ViewModelProvider;

import com.utopiaindustries.hseobservationsapp.R;
import com.utopiaindustries.hseobservationsapp.activities.DashboardActivity;
import com.utopiaindustries.hseobservationsapp.adapters.RiskLevelAdapter;
import com.utopiaindustries.hseobservationsapp.helper.Helper;
import com.utopiaindustries.hseobservationsapp.helper.Preference;
import com.utopiaindustries.hseobservationsapp.utils.ProgressDialogFragment;
import com.utopiaindustries.hseobservationsapp.utils.StorageManager.HseReportRequest;
import com.utopiaindustries.hseobservationsapp.utils.StorageManager.ObservationModel;
import com.utopiaindustries.hseobservationsapp.utils.StorageManager.StorageManager;
import com.utopiaindustries.hseobservationsapp.viewmodels.LoginViewModel;

import java.util.ArrayList;
import java.util.List;

public class ObservationTwoActivity extends AppCompatActivity {

    private Button btnNext;
    ImageView imgBack;

    private AutoCompleteTextView riskLevelTextview;

    private RiskLevelAdapter riskLevelAdapter;

    private ArrayList<String> riskLevelArrayList = new ArrayList<>();
    String riskLevel = "";
    RadioGroup rg1;
    RadioButton rb_open, rb_closed;
    EditText etDescription;
    String obStatus = "";
    LoginViewModel loginViewModel;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        setContentView(R.layout.activity_observation_two);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });

        initializeLayout();

        imgBack.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        btnNext.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isValidate()) {
                    alertReportSubmit(ObservationTwoActivity.this);
                }

            }
        });

        riskLevelTextview.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {

                riskLevel = riskLevelArrayList.get(position);
                StorageManager.getInstance().getObservationsModel().get(0).setRiskLevel(riskLevelArrayList.get(position));

            }
        });

        etDescription.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                StorageManager.getInstance().getObservationsModel().get(0).setDescription(s.toString());
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        rg1.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener()
        {
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                if (checkedId == R.id.rb_open) {
                    obStatus = "true";
                    StorageManager.getInstance().getObservationsModel().get(0).setObservationStatus(true);
                } else if (checkedId == R.id.rb_closed) {
                    obStatus = "false";
                    StorageManager.getInstance().getObservationsModel().get(0).setObservationStatus(false);
                }
            }
        });
    }

    private void initializeLayout() {

        btnNext = findViewById(R.id.btn_next);
        imgBack = findViewById(R.id.img_back);
        rg1 = findViewById(R.id.rg1);
        etDescription = findViewById(R.id.et_description);
        etDescription.setImeOptions(EditorInfo.IME_ACTION_DONE);
        etDescription.setRawInputType(InputType.TYPE_CLASS_TEXT);

        rb_open = findViewById(R.id.rb_open);
        rb_closed = findViewById(R.id.rb_closed);

        riskLevelTextview = findViewById(R.id.risk_textview);

        riskLevelArrayList.add("High");
        riskLevelArrayList.add("Medium");
        riskLevelArrayList.add("Low");

        riskLevelAdapter = new RiskLevelAdapter(this, riskLevelArrayList);
        riskLevelTextview.setAdapter(riskLevelAdapter);

        loginViewModel = new ViewModelProvider(this).get(LoginViewModel.class);

        loginViewModel.getLoadingState().observe(this, isLoading -> {

            if (isLoading != null && isLoading) {
                showProgressDialog();
            } else {
                dismissProgressDialog();
            }
        });

        loginViewModel.getErrorMessage().observe(this, errorResponse -> {
            if (errorResponse.isEmpty()) {
                Toast.makeText(this, "Something went wrong", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(this, errorResponse, Toast.LENGTH_SHORT).show();
            }
        });

        loginViewModel.getUserSaveLiveData().observe(this, hseSaveResponse -> {
            Log.e("Response: ",""+hseSaveResponse);
            if (hseSaveResponse != null && hseSaveResponse.getResult().equalsIgnoreCase("success")) {
                Toast.makeText(this, "Report Submitted Successfully", Toast.LENGTH_SHORT).show();
                StorageManager.getInstance().clearObservationModule();
                Intent intent = new Intent(ObservationTwoActivity.this, DashboardActivity.class);
                intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                startActivity(intent);
                finish();
                overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left);


            }
        });
    }

    public void alertReportSubmit(Context con) {
        ViewGroup viewGroup = findViewById(android.R.id.content);

        TextView dialogOkBtn, dialogCancelBtn;

        AlertDialog.Builder builder = new AlertDialog.Builder(con);
        View view1 = LayoutInflater.from(con).inflate(R.layout.custom_layout_for_report_submission, viewGroup, false);
        builder.setCancelable(false);
        builder.setView(view1);

        dialogOkBtn = view1.findViewById(R.id.dialogOkBtn);
        dialogCancelBtn = view1.findViewById(R.id.dialogCancelBtn);

        AlertDialog alertDialog = builder.create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        dialogOkBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                alertDialog.dismiss();

                List<ObservationModel> observationList = StorageManager.getInstance().getObservationsModel();
                HseReportRequest<ObservationModel> observationRequest = new HseReportRequest<>();
                observationRequest.setObservation_date(StorageManager.getInstance().getObservation_date());
                observationRequest.setUserId(StorageManager.getInstance().getUserId());
                observationRequest.setSiteId(StorageManager.getInstance().getSite_id());
                observationRequest.setSupervisorId(StorageManager.getInstance().getSupervisorId());
                observationRequest.setSupervisorName(StorageManager.getInstance().getSupervisorName());
                observationRequest.setShift(StorageManager.getInstance().getShift());
                observationRequest.setDepartmentId(StorageManager.getInstance().getDepartmentId());
                observationRequest.setBuildingId(StorageManager.getInstance().getBuildingId());
                observationRequest.setFloorId(StorageManager.getInstance().getFloorId());
                observationRequest.setRecordTypeId(StorageManager.getInstance().getRecordTypeId());
                observationRequest.setCreatedBy(Preference.getMyStringPref(Helper.project_file, Helper.logInUser, ObservationTwoActivity.this));
                observationRequest.setAppVersion(StorageManager.getInstance().getAppVersion());
                observationRequest.setReportData(observationList);

                /*Gson gson = new GsonBuilder().setPrettyPrinting().create();
                String jsonRequest = gson.toJson(observationRequest);
                Log.e("RequestModel JSON", jsonRequest);*/

                loginViewModel.saveHSEData(observationRequest);
            }
        });

        dialogCancelBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                alertDialog.dismiss();
                //Toast.makeText(con, "Cancel", Toast.LENGTH_SHORT).show();

            }
        });

        alertDialog.show();
    }

    public boolean isValidate() {
        boolean returnValue = true;
        String message = "";

        if (etDescription.getText().toString().isEmpty()) {
            message = "Please enter description.";
            returnValue = false;
        }

        if (obStatus.isEmpty()) {
            message = "Please select Status.";
            returnValue = false;
        }

        if (riskLevel.isEmpty()) {
            message = "Please select Risk level description.";
            returnValue = false;
        }

        if (!returnValue) {
            Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
        }

        return returnValue;
    }

    public void showProgressDialog() {
        ProgressDialogFragment progressDialog = new ProgressDialogFragment();
        progressDialog.setCancelable(false);
        progressDialog.show(getSupportFragmentManager(), "progressDialog");
    }

    public void dismissProgressDialog() {
        ProgressDialogFragment progressDialog = (ProgressDialogFragment)
                getSupportFragmentManager().findFragmentByTag("progressDialog");
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
    }

    @Override
    protected void onResume() {
        super.onResume();

        if (!StorageManager.getInstance().getObservationsModel().isEmpty()) {
            showPersistData();
        }
    }

    private void showPersistData() {
        riskLevelTextview.post(() -> riskLevelTextview.setText(StorageManager.getInstance().getObservationsModel().get(0).getRiskLevel(), false));

        if(StorageManager.getInstance().getObservationsModel().get(0).isObservationStatus()) {
            rb_open.setChecked(true);
        }
        else if (!StorageManager.getInstance().getObservationsModel().get(0).isObservationStatus()) {
            rb_closed.setChecked(true);
        }

        etDescription.setText(StorageManager.getInstance().getObservationsModel().get(0).getDescription());

    }
}